/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2016 Richard Hughes <richard@hughsie.com>
 *
 * SPDX-License-Identifier: GPL-2.0+
 */

#pragma once

#ifndef I_KNOW_THE_GNOME_SOFTWARE_API_IS_SUBJECT_TO_CHANGE
#error You have to define I_KNOW_THE_GNOME_SOFTWARE_API_IS_SUBJECT_TO_CHANGE
#endif

#include <gs-app.h>
#include <gs-app-list.h>
#include <gs-app-collation.h>
#include <gs-autocleanups.h>
#include <gs-category.h>
#include <gs-metered.h>
#include <gs-os-release.h>
#include <gs-plugin.h>
#include <gs-plugin-vfuncs.h>
#include <gs-utils.h>
